{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit FraWebCoreProjectOptions;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, typinfo,
  // LazUtils
  LazVersion, LazLoggerBase,
  // LCL
  StdCtrls,
  // IdeIntf
  LazIDEIntf, ProjectIntf, CompOptsIntf, IDEOptEditorIntf, IDEOptionsIntf;

type

  { TWebCoreProjectOptionsFrame }

  TWebCoreProjectOptionsFrame = class(TAbstractIDEOptionsEditor)
    BResetRunCommand: TButton;
    BResetCompileCommand: TButton;
    CBOpenBrowserPage: TCheckBox;
    CBRunOnReady: TCheckBox;
    CBSingleJS: TCheckBox;
    CBStartWebServer: TCheckBox;
    CBWebProject: TCheckBox;
    CBHTMLFile: TComboBox;
    LCBProjectHTMLFile: TLabel;
    procedure BResetCompileCommandClick(Sender: TObject);
    procedure BResetRunCommandClick(Sender: TObject);
    procedure CBWebProjectChange(Sender: TObject);
  private
    FOldWebProject: String;
    FDialog: TAbstractOptionsEditorDialog;
    function FillFilesCombo(Prj: TLazProject): Integer;
    procedure UpdateEnabled;
  public
    function GetTitle: string; override;
    procedure Setup({%H-}ADialog: TAbstractOptionsEditorDialog); override;
    procedure ReadSettings({%H-}AOptions: TAbstractIDEOptions); override;
    procedure WriteSettings({%H-}AOptions: TAbstractIDEOptions); override;
    procedure RestoreSettings({%H-}AOptions: TAbstractIDEOptions); override;
    class function SupportedOptionsClass: TAbstractIDEOptionsClass; override;
  end;

Procedure SetDefaultWebCoreProjectRunParams(aProject: TLazProject);
Procedure SetDefaultWebCoreRunParams(RunParams : TAbstractRunParamsOptionsMode);
Procedure SetDefaultWebCoreCompileOptions(CompOpts: TLazCompilerOptions);

implementation

uses TMSWebCoreIDEOptions;

procedure SetDefaultWebCoreProjectRunParams(aProject: TLazProject);
const
  DefaultRunParamMode = 'Default';
var
  RunParams: TAbstractRunParamsOptions;
begin
  RunParams:=aProject.RunParameters;
  SetDefaultWebCoreRunParams(RunParams.GetOrCreate(DefaultRunParamMode));
  if RunParams.ActiveModeName='' then
    RunParams.ActiveModeName:=DefaultRunParamMode;
end;

Procedure SetDefaultWebCoreRunParams(RunParams: TAbstractRunParamsOptionsMode);
begin
  RunParams.HostApplicationFilename:='';
  RunParams.CmdLineParams:='';
  RunParams.UseLaunchingApplication:=True;
  RunParams.LaunchingApplicationPathPlusParams:=WebCoreDefaultLaunchingAppPlusParams;
end;

Procedure SetWebCoreCompileOptions(CompOpts: TLazCompilerOptions;
  TargetOS, CustomOpts : String);
begin
  DebugLn(['Hint: (lazarus) [SetPasJSCompileOptions] START']);
  // set CompilerPath early, so that other macros do not trigger a compiler scan
  CompOpts.CompilerPath:=WebCoreDefaultPas2js;
  CompOpts.UnitOutputDirectory:='js';
  CompOpts.TargetOS:=TargetOS;
  CompOpts.AllowLabel:=false;
  CompOpts.UseAnsiStrings:=false;
  CompOpts.IOChecks:=false;
  CompOpts.StackChecks:=false;
  CompOpts.SmartLinkUnit:=false;
  CompOpts.GenerateDebugInfo:=true; // generate js.map
  CompOpts.DebugInfoType:=dsAuto;
  CompOpts.UseLineInfoUnit:=false;
  CompOpts.UseHeaptrc:=false;
  CompOpts.Win32GraphicApp:=false;
  CompOpts.WriteFPCLogo:=true;
  CompOpts.CustomOptions:=CustomOpts;
  CompOpts.UnitOutputDirectory:=WebCoreDefaultOutputPath;
  CompOpts.TargetFilenameApplyConventions:=False;
  CompOpts.TargetFileExt:='.js';
  debugln(['Hint: (lazarus) [SetWebCoreCompileOptions] Compiler=',CompOpts.CompilerPath,' TargetOS=',CompOpts.TargetOS,' Custom="',CompOpts.CustomOptions,'"']);
  CompOpts.SetAlternativeCompile('"$(WebCoreCompiler)" $(WebCoreCompilerOpts)',True);
  // ToDo: set  CompOpts.ExecuteBefore.Parsers to pas2js;
  {$IF laz_fullversion>2000009}
  CompOpts.ExecuteBefore.Parsers.Text:='pas2js';
  {$ENDIF}
  // set pascal compiler to TMS pas2js.exe, needed by IDE to query macros for codetools
  CompOpts.CompilerPath:=WebCoreDefaultPas2js; // set again, because SetAlternativeCompile cleared it
  CompOpts.CustomOptions:='-Jc -Jirtl.js';
  {$if laz_fullversion<2000100}
  // Lazarus 2.0.0 requires that the compiler is either executed on Run or Build
  SetOrdProp(CompOpts,'CompileReasons',2);
  CompOpts.CustomOptions:=CompOpts.CustomOptions+' PleaseUseCompileNotBuild';
  {$endif}
  {$if (laz_fullversion<2010000) and defined(darwin)}
  // Lazarus 2.0.x on darwin checks type of debugger info -> not needed for pas2js => set something to keep the IDE happy
  CompOpts.DebugInfoType:=dsDwarf3;
  {$endif}
end;

Procedure SetDefaultWebCoreCompileOptions(CompOpts: TLazCompilerOptions);
begin
  // Adjust as needed !
  SetWebCoreCompileOptions(CompOpts,'browser','');
end;

{$R *.lfm}

{ TWebCoreProjectOptionsFrame }

function TWebCoreProjectOptionsFrame.GetTitle: string;
begin
  Result:='Web Project (TMS WebCore)';
end;

procedure TWebCoreProjectOptionsFrame.Setup(ADialog: TAbstractOptionsEditorDialog);
begin
  // Do nothing
  FDialog:=ADialog;
end;

procedure TWebCoreProjectOptionsFrame.BResetRunCommandClick(Sender: TObject);
begin
  SetDefaultWebCoreProjectRunParams(LazarusIDE.ActiveProject);
end;

procedure TWebCoreProjectOptionsFrame.CBWebProjectChange(Sender: TObject);
begin
  UpdateEnabled;
end;

procedure TWebCoreProjectOptionsFrame.BResetCompileCommandClick(Sender: TObject);
Var
  Prj : TLazProject;
begin
  PRJ:=LazarusIDE.ActiveProject;
  SetDefaultWebCoreCompileOptions(PRJ.LazCompilerOptions);
end;

// Fill combo with HTM(L) files.
// Return index of file that has IsProjectHTMLFile set.
function TWebCoreProjectOptionsFrame.FillFilesCombo(Prj: TLazProject): Integer;
Var
  I : integer;
  HtmlPF,PF : TLazProjectFile;
  Ext : String;
  L : TStringList;
begin
  Result:=-1;
  HtmlPF:=Nil;
  L:=TStringList.Create;
  try
    For I:=0 to Prj.FileCount-1 do
      begin
      PF:=Prj.Files[i];
      Ext:=LowerCase(ExtractFileExt(PF.Filename));
      if Ext='.html' then
        begin
        L.AddObject(PF.FileName,PF);
        If (HtmlPF=nil) and (PF.CustomData[KeyIsProjectHTMLFile]='1') then
          HtmlPF:=PF;
        end;
      end;
    L.Sort;
    CBHTMLFile.Items:=L;;
  finally
    L.Free;
  end;
  if (HtmlPF<>Nil) then
    Result:=CBHTMLFile.Items.IndexOfObject(HtmlPF);
end;

procedure TWebCoreProjectOptionsFrame.UpdateEnabled;
var
  Enable: Boolean;
begin
  Enable:=CBWebProject.Checked;
  BResetRunCommand.Enabled:=Enable;
  BResetCompileCommand.Enabled:=Enable;
  CBRunOnReady.Enabled:=Enable;
  CBSingleJS.Enabled:=Enable;
  CBStartWebServer.Enabled:=Enable;
  CBHTMLFile.Enabled:=Enable;
  LCBProjectHTMLFile.Enabled:=Enable;
end;

procedure TWebCoreProjectOptionsFrame.ReadSettings(AOptions: TAbstractIDEOptions);

Var
  Prj : TLazProject;
  HTMLIdx : Integer;
begin
  Prj:=LazarusIDE.ActiveProject;
  if Prj=nil then exit;
  FOldWebProject:=Prj.CustomData[KeyTMSWebCoreProject];
  CBWebProject.Checked:=Prj.CustomData[KeyTMSWebCoreProject]='1';
  UpdateEnabled;

  HTMLIdx:=FillFilesCombo(Prj);
  CBHTMLFile.ItemIndex:=HTMLIdx;

  if Prj.CustomData.Contains(KeyProjectRunOnReady) then
    CBRunOnReady.Checked:=Prj.CustomData[KeyProjectRunOnReady]='1'
  else
    CBRunOnReady.Checked:=WebCoreDefaultRunOnReady;

  if Prj.CustomData.Contains(KeySingleJS) then
    CBSingleJS.Checked:=Prj.CustomData[KeySingleJS]='1'
  else
    CBSingleJS.Checked:=WebCoreOptions.SingleJSFile;

  if Prj.CustomData.Contains(KeyStartWebServer) then
    CBStartWebServer.Checked:=Prj.CustomData[KeyStartWebServer]='1'
  else
    CBStartWebServer.Checked:=WebCoreDefaultStartWebServer;

  if Prj.CustomData.Contains(KeyRunWebRunner) then
    CBOpenBrowserPage.Checked:=Prj.CustomData[KeyRunWebRunner]='1'
  else
    CBOpenBrowserPage.Checked:=WebCoreDefaultRunWebRunner;
end;

procedure TWebCoreProjectOptionsFrame.WriteSettings(AOptions: TAbstractIDEOptions);

Var
  Prj : TLazProject;

  Procedure DoBool(N : String; AValue, DefaultValue : Boolean);
  begin
    if AValue=DefaultValue then
      Prj.CustomData.Remove(N)
    else
      Prj.CustomData[N]:=BoolToStr(AValue,'1','0');
  end;

var
  PrjFile, HtmlFile: TLazProjectFile;
  i: Integer;
begin
  Prj:=LazarusIDE.ActiveProject;
  if Prj=nil then exit;
  // Change what is needed
  DoBool(KeyTMSWebCoreProject,CBWebProject.Checked,false);
  if CBWebProject.Checked then
    begin
    HtmlFile:=nil;
    with CBHTMLFile do
      if ItemIndex>=0 then
        HtmlFile:=Items.Objects[ItemIndex] as TLazProjectFile;
    for i:=0 to Prj.FileCount-1 do
      begin
      PrjFile:=Prj.Files[i];
      if PrjFile=HtmlFile then
        PrjFile.CustomData[KeyIsProjectHTMLFile]:='1'
      else
        PrjFile.CustomData.Remove(KeyIsProjectHTMLFile);
      end;
    DoBool(KeyProjectRunOnReady,CBRunOnReady.Checked,WebCoreDefaultRunOnReady);
    DoBool(KeySingleJS,CBSingleJS.Checked,WebCoreOptions.SingleJSFile);
    DoBool(KeyStartWebServer,CBStartWebServer.Checked,WebCoreDefaultStartWebServer);
    DoBool(KeyRunWebRunner,CBOpenBrowserPage.Checked,WebCoreDefaultRunWebRunner);
    end
  else
    // Clear everything
    With Prj.CustomData do
      begin
      Remove(KeyProjectRunOnReady);
      Remove(KeySingleJS);
      Remove(KeyStartWebServer);
      Remove(KeyRunWebRunner);
      end;
end;

procedure TWebCoreProjectOptionsFrame.RestoreSettings(
  AOptions: TAbstractIDEOptions);
var
  Prj: TLazProject;
begin
  Prj:=LazarusIDE.ActiveProject;
  if Prj=nil then exit;
  if FOldWebProject='' then
    Prj.CustomData.Remove(KeyTMSWebCoreProject)
  else
    Prj.CustomData[KeyTMSWebCoreProject]:=FOldWebProject;
end;

class function TWebCoreProjectOptionsFrame.SupportedOptionsClass: TAbstractIDEOptionsClass;
begin
  Result:=TAbstractIDEProjectOptions;
end;

end.

